#!/usr/bin/env Rscript

library("argparse")

#---- find the directory of the current script
initial.options <- commandArgs(trailingOnly = FALSE)
file.arg.name <- "--file="
script.name <- sub(file.arg.name, "", initial.options[grep(file.arg.name, initial.options)])
script.dirname <- dirname(script.name)
#----

source(file.path(script.dirname, "functions.R"))

# some parameters
fields_separator <- " "

# Parse arguments
parser <-
  ArgumentParser(description = "Build a Volcano plot from data generated by 'tradis_comparison.R' script")

parser$add_argument("-w", "--wig",
  type = "character",
  help = "Wiggle file", required = T
)
parser$add_argument("-o", "--out",
  type = "character",
  help = "The path of the file used to write the normalized wiggle file", required = T
)
args <- parser$parse_args()

# Parse wig file
data <- parse_wig(args$wig)

# Perform prediction and normalization on the wig file
pred <- perform_loess_prediction(data)
data.norm <- perform_loess_normalization(data, pred)

# Write the noramlized file
write_wig(data.norm, args$out, fields_separator)


print(paste("Apply loess normalization on:", args$wig))
print(paste("\tTotal count before normalization:", sum(data$TA_counts)))
print(paste("\tTotal count after normalization:", sum(data.norm$TA_counts)))
