# This module is not intended to be reusable.
# This module is intended to avoid namespace collision between parameters in different part of the app (process-read, core..)

# UI
createDatabaseUI <- function(id) {
  ns <- NS(id)
  fluidRow(
    box(
      width = 12,
      title = "Inputs",
      status = "primary",
      tags$p("Select your project directory (required):"),
      inputDirUI(ns("project-dir")),
      br(),
      tags$p("Enter your input file (orthology table, TSV format, required):"),
      inputFileUI(ns("ortho-table"))
    ),
    uiOutput(ns("create_db_ui"))
  )
}

# Server
createDatabaseServer <- function(id) {
  moduleServer(id, function(input, output, session) {
    inputs_dir_path <- inputDirServer("project-dir")
    ortho_table_path <- inputFileServer("ortho-table")
    outdir <- inputs_dir_path
    command <- reactive({
      inputs_dir_path_val <- inputs_dir_path()
      ortho_table_path_val <- ortho_table_path()
      database<- file.path(inputs_dir_path_val, 'data.db')
      glue(
        "tnphyto.py create-db -o \"{database}\" \n",
        "-p \"{inputs_dir_path_val}\" \n",
        "-g \"{ortho_table_path_val}\" \n",
        "--force")
    })

    output$command <- renderText(
      command()
    )
    runBackgroundJobServer("create-db", command, outdir, not_ok_if_outdir_exists=F)

    output$create_db_ui <- renderUI({
      req(inputs_dir_path(), ortho_table_path())
      tagList(
        box(
          width = 12,
          title = "Command",
          status = "primary",
          collapsible = T,
          collasped = T,
          verbatimTextOutput(session$ns("command")),
          runButtonUI(session$ns("create-db"), "Create database"),
        ),
        box(
          width = 12,
          title = "Logs",
          status = "primary",
          collapsible = T,
          collapsed = T,
          runLogUI(session$ns("create-db"))
        )
      )
    })
  })
}
