# UI
inputFileUI <- function(id) {
  ns <- NS(id)
  div(
    class = "input-group",
    span(
      class = "input-group-btn",
      shinyFilesButton(
        ns("file"),
        label = "Browse",
        title = "Please select a file",
        multiple = F
      )
    ),
    textOutput(ns("path")) %>%
      tagAppendAttributes(class = "form-control")
  )
}

inputDirUI <- function(id) {
  ns <- NS(id)
  div(
    class = "input-group",
    span(
      class = "input-group-btn",
      shinyDirButton(
        ns("dir"),
        label = "Browse",
        title = "Please select a project directory",
      )
    ),
    textOutput(ns("path")) %>%
      tagAppendAttributes(class = "form-control")
  )
}

# Server
inputFileServer <- function(id, filetypes = c("tsv")) {
  moduleServer(id, function(input, output, session) {
    # ns <- session$ns
    
    path <- reactiveVal(NULL)
    
    output$path <- renderText({
      if (is.null(path())) {
        "No file have been selected"
      } else {
        fs::path_rel(path(), start = PLAYGROUND)
      }
    })
    
    observeEvent(input$file, {
      val <- parseFilePaths(volumes, input$file) %>%
        slice(n()) %>%
        select(datapath) %>%
        paste()
      if (file.exists(val)) {
        path(val)
      }
      
    })
    
    volumes <-
      c(Home = fs::path_home(),
        "R Installation" = R.home(),
        getVolumes()())
    
    shinyFileChoose(
      input,
      id = "file",
      roots = volumes,
      session = session,
      filetypes = filetypes
    )
    
    path
  })
}

inputDirServer <- function(id) {
  moduleServer(id, function(input, output, session) {
    # ns <- session$ns
    
    path <- reactiveVal(NULL)
    
    output$path <- renderText({
      if (is.null(path())) {
        "No dir have been selected"
      } else {
        fs::path_rel(path(), start = PLAYGROUND)
      }
    })
    
    observeEvent(input$dir, {
      val <- parseDirPath(volumes, input$dir)

      if (!rlang::is_empty(val) && dir.exists(val)) {
        path(val)
      }
      
    })
    
    volumes <-
      c(Home = fs::path_home(),
        "R Installation" = R.home(),
        getVolumes()())
    
    shinyDirChoose(
      input,
      id = "dir",
      roots = volumes,
      session = session
    )
    
    path
  })
}
