# This module is not intended to be reusable.
# This module is intended to avoid namespace collision between parameters in different part of the app (process-read, core..)


# FUNCTIONS
# TODO vérifier que le mean se fait bien rowwize



bin_wiggle_path <- function(path, bin_width) {
  df <- parse_wig(path) %>% arrange(TA_pos)
  bin_wiggle_df(df, bin_width)
}

bin_wiggle_df <- function(df, bin_width) {
  #df <- data.frame(TA_pos = sample(1:1000, 40, replace=F), TA_counts=runif(40)) %>% arrange(TA_pos)
  #df <- data.frame(TA_pos = seq(40), TA_counts=runif(40)) %>% arrange(TA_pos)
  brks = seq(0, max(df$TA_pos) + bin_width, bin_width)
  df %>% mutate(bin_idx = findInterval(TA_pos, brks),
                bin_midpoint = ((brks[bin_idx] + brks[bin_idx + 1]) / 2)) %>% 
    group_by(bin_midpoint) %>% summarize(bin_counts = mean(TA_counts))
}


generate_genome_wide_viz_fast <- function(df, title, xcol="TA_pos", ycol="TA_counts", span=1) {
  p <- ggplot(df, aes(x = .data[[xcol]], y = .data[[ycol]])) +
    geom_point(shape = 20, col = "steelblue") +
    geom_smooth(method = "loess", se = FALSE, span=span, colour="#E69F00") + 
    labs(
      title = title,
      x = "Positions on genome (bp)",
      y = "# of Insertions"
    )
  
  plotly::toWebGL(p)
  
}


# UI
normalizationUI <- function(id) {
  ns <- NS(id)
  fluidRow(
    box(
      width = 12,
      title = "Inputs",
      status = "primary",
      
      #tags$p('This module is intended to works on RAW wiggle files only.', class="text-danger"),
      tags$p("Enter your input file (TSV file with raw count & annotations):"),
      # In order to align the shinyFilesButton with the textOutput on the same line use this div class.
      inputFileUI(ns("inputs-table"))
    ),
    uiOutput(ns("normalization_ui"))
  )
}


# Server
normalizationServer <- function(id) {
  moduleServer(id, function(input, output, session) {
    inputs_table_path <- inputFileServer("inputs-table")
    
    project_name <- reactive({
      req(inputs_table_path())
      fs::path_split(fs::path_rel(inputs_table_path(), start = PLAYGROUND))[[1]][1]
    })
    
    outdir <- reactive({
      outdir <- file.path(PLAYGROUND, project_name(), "normalize")
    })
    
    inputs_table_df <- reactive({
      req(inputs_table_path())

      inputs_table_dir <- dirname(inputs_table_path())
      df <- read_delim(inputs_table_path(), 
                       delim = "\t", escape_double = FALSE, 
                       col_types = cols(rep = col_integer()), 
                       trim_ws = TRUE) %>% select(c(genus, strain, condition, rep, pool, wiggle, genbank)) %>%
        mutate(wiggle=normalizePath(file.path(inputs_table_dir, wiggle)),
               genbank=normalizePath(file.path(inputs_table_dir, genbank))) # we use abs path internally
      df
    })
    
    inputs_file_for_normalization_step <- tempfile(fileext='.tsv')
    
    strains <- reactive({
      req(inputs_table_df())
      inputs_table_df() %>% distinct(strain) %>% arrange(strain) %>% pull(strain)
    })
    
    output$table_for_genome_wild_viz <- renderDataTable({
      req(inputs_table_df())
      inputs_table_df() %>%
        mutate(wiggle=fs::path_rel(wiggle, start = PLAYGROUND)) %>% # for the user, we display relative path
        select(-genbank) %>% # hide irrelevant col
        DT::datatable(options = list(lengthMenu = c(5, 10, 15), pageLength = 5, scrollX = TRUE), selection='single')
    })
    
    
    # output$raw_genome_wide_viz <- renderDataTable({
    #   req(input$table_for_genome_wild_viz_rows_selected)
    #   idx <- input$table_for_genome_wild_viz_rows_selected
    #   
    #   if (length(idx) != 1)
    #     return(NULL)
    #   wiggle <- inputs_table_df() %>% slice(idx) %>% pull(wiggle)
    #   generate_genome_wide_table(wiggle)
    # })
    
    output$genome_wide_viz <- renderPlotly({
      req(input$table_for_genome_wild_viz_rows_selected)
      idx <- input$table_for_genome_wild_viz_rows_selected
      
      if (length(idx) != 1)
        return(NULL)
      wiggle_path <- inputs_table_df() %>% slice(idx) %>% pull(wiggle)
      
      bin_width = 10000
      bin_table <- bin_wiggle_path(wiggle_path, bin_width)
      relative_wiggle_path = fs::path_rel(wiggle_path, start = PLAYGROUND)
      plot_title = glue("Mean insertion (per {bin_width} bp) on \n    {relative_wiggle_path}")
      generate_genome_wide_viz_fast(bin_table, title=plot_title, xcol="bin_midpoint", ycol="bin_counts")
    }) 
    #%>% bindCache(   input$table_for_genome_wild_viz_rows_selected,
    #                 cache = "session")
    command <- reactive({
      inputs_table_path_val <- inputs_file_for_normalization_step
      outdir_val <- outdir()
      
      verbose_option <- ''
      if (input$verbose) {
        verbose_option <- '--verbose'
      }
      
      glue(
        "tnphyto.py normalize -i \"{inputs_table_path_val}\" -o \"{outdir_val}\" {verbose_option}"
      )
    })
    
    output$command <- renderText(
      command()
    )
    runBackgroundJobServer("normalize", command, outdir)
    
    observeEvent(input$save_table_and_run, {
      # basically, append two columns of booleans (loess & ttr) to reflect the choices
      # of the user
      inputs_table_for_normalization_step <- inputs_table_df() %>% 
        mutate(loess = ifelse(strain %in% input$loess, TRUE, FALSE)) %>%
        mutate(ttr = input$ttr)
      
      # write the table in a temp dir
      write_tsv(inputs_table_for_normalization_step, file=inputs_file_for_normalization_step)
      # then, simulate a click on runButton to excecute the command
      click(session$ns("run"))
    })
    
    
    output$normalization_ui <- renderUI({
      # This UI will be rendered when the inputs are set by the user
      # Basically, it contains the parameters used for the analysis and a button to run it.
      req(inputs_table_path())
      tagList(
        box(
          collapsible = T,
          width = 12,
          status = "info",
          title = "LOESS normalization:",
          tags$p('Select a dataset in the table bellow to show the distribution of TA sites along the genome.'),
          dataTableOutput(session$ns('table_for_genome_wild_viz')),
        ),
        box(
          collapsible = T,
          collapsed = T,
          width = 12,
          title = "RAW genome wide visualization",
          status = "info",
          plotlyOutput(session$ns("genome_wide_viz"), height = '800'),
        ),
        
        
        box(
          collapsible = T,
          width = 12,
          title = "LOESS normalization: select strains",
          status = "info",
          tags$p("Select strains for which you whant to apply LOESS normalization"),
          tagList(
            checkboxGroupInput(session$ns('loess'), label="", choices = strains(), selected = NULL)
          )
        ),
        box(
          collapsible = T,
          width = 12,
          title = "TTR normalization",
          status = "info",
          tagList(
            checkboxInput(session$ns("ttr"), label="Apply to all datasets", value = TRUE)
          )
        ),
        box(
          collapsible = T,
          width = 12,
          title = "Verbosity",
          status = "info",
          tagList(
            checkboxInput(
              session$ns("verbose"),
              label="verbose"
            )
          )
      ),
        box(
          collapsible = T,
          collapsed = F,
          width = 12,
          title = "Command",
          status = "info",
          verbatimTextOutput(session$ns("command")),
          actionButton(session$ns("save_table_and_run"), "Compute normalization"),
          # hide the runButton as it will be triggered by the previous action button
          # after the saving of the table
          div(class="invisible",
             runButtonUI(session$ns("normalize"), "Compute normalization"))
          ),
          box(
            width = 12,
            title = "Logs",
            status = "primary",
            collapsible = T,
            collapsed = T,
            runLogUI(session$ns("normalize"))
          )
        )
        
    })
    list(
      inputs_table = inputs_table_path, # to be able to set this value from outside
      outdir = outdir,
      next_steps = c("core_genome", "condi_analysis")
    )
  })
}
